!   This simulation routine works with InitializeAggregateShockSeries() 
!   that generates aggregate productivity shocks on pgrid.

subroutine Simulation()

	use Globals
	use LinInterpModule
	use PolyInterpModule
	use dflib
	use imsl_libraries
	use rnunf_int
	use rnnor_int
	use csint_int
	use csval_int


	implicit none


	integer, parameter:: xseed = 2
	real(8):: theta_pred, pth_pred, qth_fe, theta_fe, VM, K_curr, K_next
	real(8):: maxJ(na,nx,np), EmaxJGH(na,nk,np), EmaxJGHkp(na)
	real(8):: Wkp(na,nx), Ukp(na), AEkp(na,nx), AUkp(na), Wagekp(na,nx)
	real(8):: break_EmaxJ(na), cscoef_EmaxJ(4,na), break_wage(na,nx), cscoef_wage(4,na,nx), &
	           break_Wkp(na,nx), cscoef_Wkp(4,na,nx), break_Ukp(na), cscoef_Ukp(4,na)
	real(8), allocatable:: lambdavec(:), pthvec(:)
	real(8):: cutoff_lambda, cutoff_pth, xshock
	integer:: ind_E, ind_U, Nind_E, Nind_U
	type Individual
		real(8):: A, AP, W, C, lambdashock, pthshock
		integer:: E, EP, X, XP
	end type Individual
	type(Individual), dimension(Nindiv):: Worker


	!	expected value of matched firm: EmaxJGH(a,K,p), used for evaluating the free-entry condition

	EmaxJGH = 0.0D0

	do ip = 1, np
	do ik = 1, nk
	
		!	polynomial interpolate J at kp(ik,ip), and compute max{J,0} in the next period

		do ia = 1, na
			
			do jp = 1, np
			do jx = 1, nx
				maxJ(ia,jx,jp) = max(polyinterp1(kp(ik,ip), kgrid, J(ia,jx,:,jp)), 0.0D0)
			end do
			end do
		

			!	EmaxJGH(ia,ik,ip)

			do jp = 1, np
			do jx = 1, nx
				EmaxJGH(ia,ik,ip) = EmaxJGH(ia,ik,ip) + maxJ(ia,jx,jp)*prX(jx)*trP(ip,jp)
			end do
			end do

		end do

	end do
	end do


	!	set the seed number for random number generator

	call rnset(xseed)


	!	initializae asset holdings

	call d_rnnor(Worker%A)
	call dscal(Nindiv, std_asset_ss, Worker%A, 1)
	call dadd(Nindiv, ave_asset_ss, Worker%A, 1)
    Worker%A = min(max(Worker%A, agrid(1)), agrid(na))


    !   initialize employment status
    
    do indiv = 1, Nindiv
    
        xshock = d_rnunf()
        if (xshock <= 0.94D0) then
            Worker(indiv)%E = 1
        else
            Worker(indiv)%E = 0
        end if
        
    end do
    

    !   initialize productivity
    
	do indiv = 1, Nindiv
	
	    if (Worker(indiv)%E == 1) then
	    
	        xshock = d_rnunf()
    		do ix = 1, nx
                if (xshock <= CprX(ix)) then
                    Worker(indiv)%X = ix
                    exit
                end if
            end do

        end if

	end do


	!	start simulation 

	do time = 1, Nsim


		!	allocate memory for random numbers
		
		Nind_E = sum(Worker%E,mask=Worker%E==1)
		Nind_U = sum(1-Worker%E,mask=Worker%E==0)

		allocate(lambdavec(Nind_E), pthvec(Nind_U))


		!	average capital of the unemployed, predicted theta and worker's matching probability
        !   this version uses average capital of the unemployed, 
        !   but late try aggregate capital of the entire economy
        
		K_curr = sum(Worker%A,mask=Worker%E==0)/Nind_U
		theta_pred = dexp(coef_th(1) + coef_th(2)*dlog(K_curr) + coef_th(3)*dlog(Pdata(time)))
		pth_pred = match_scale*theta_pred**match_pow


		!	interpolate asset decision rule at K_curr and Pdata(time)

		do ia = 1, na
			do ix = 1, nx
				AEkp(ia,ix) = polyinterp1(K_curr, kgrid, AE(ia,ix,:,iPdata(time)))
			end do
			AUkp(ia) = polyinterp1(K_curr, kgrid, AU(ia,:,iPdata(time)))
		end do


		!	interpolate wages at Kdata(time) and Pdata(time)

		do ia = 1, na
		do ix = 1, nx
			Wagekp(ia,ix) = polyinterp1(K_curr, kgrid, Wage(ia,ix,:,iPdata(time)))
		end do
		end do

		do ix = 1, nx
			call d_csint(agrid, Wagekp(:,ix), break_wage(:,ix), cscoef_wage(:,:,ix))
		end do


		!	individual's asset holdings in the next period, assign random number for next period match status

		ind_E = 0
		ind_U = 0

		do indiv = 1, Nindiv

			if (Worker(indiv)%E == 1) then

				Worker(indiv)%W = d_csval(Worker(indiv)%A, break_wage(:,Worker(indiv)%X),	&
										  cscoef_wage(:,:,Worker(indiv)%X))
				Worker(indiv)%AP = lininterp1(Worker(indiv)%A, agrid, AEkp(:,Worker(indiv)%X))

				ind_E = ind_E + 1
				lambdavec(ind_E) = d_rnunf()
				Worker(indiv)%lambdashock = lambdavec(ind_E)

			else

				Worker(indiv)%AP = lininterp1(Worker(indiv)%A, agrid, AUkp)

				ind_U = ind_U + 1
				pthvec(ind_U) = d_rnunf()
				Worker(indiv)%pthshock = pthvec(ind_U)

			end if

		end do


		!	sort lambdavec and pthvec

		call sortqq(loc(lambdavec), Nind_E, SRT$REAL8)
		call sortqq(loc(pthvec), Nind_U, SRT$REAL8)

		cutoff_lambda = lambdavec(int(lambda*Nind_E))
		cutoff_pth = pthvec(int(pth_pred*Nind_U))

		deallocate(lambdavec, pthvec)


		!	assign match status in the next period

		do indiv = 1, Nindiv
	
			if (Worker(indiv)%E == 1) then
				
				if (Worker(indiv)%lambdashock <= cutoff_lambda) then
					Worker(indiv)%XP = -9
				else
        	        xshock = d_rnunf()
    	        	do ix = 1, nx
                        if (xshock <= CtrX(Worker(indiv)%X,ix)) then
                            Worker(indiv)%XP = ix
                            exit
                        end if
                    end do
				end if

			else

				if (Worker(indiv)%pthshock <= cutoff_pth) then
        	        xshock = d_rnunf()
    	        	do ix = 1, nx
                        if (xshock <= CprX(ix)) then
                            Worker(indiv)%XP = ix
                            exit
                        end if
                    end do
				else
					Worker(indiv)%XP = -9
				end if

			end if

		end do


        !   compute aggregatge capital of the unemployed in the next period using prediction equation
        
		K_next = dexp(coef_K(1) + coef_K(2)*dlog(K_curr) + coef_K(3)*dlog(Pdata(time)))


		!	interpolate worker's value functions at K_next and Pdata(time+1)

		do ia = 1, na
			do ix = 1, nx
				Wkp(ia,ix) = polyinterp1(K_next, kgrid, W(ia,ix,:,iPdata(time+1)))
			end do
			Ukp(ia) = polyinterp1(K_next, kgrid, U(ia,:,iPdata(time+1)))
		end do

		do ix = 1, nx
			call d_csint(agrid, Wkp(:,ix), break_Wkp(:,ix), cscoef_Wkp(:,:,ix))
		end do
		
		call d_csint(agrid, Ukp, break_Ukp, cscoef_Ukp)
		

        !   worker's employment decision in the next period

        do indiv = 1, Nindiv
        
            if (Worker(indiv)%XP > 0) then

                if (d_csval(Worker(indiv)%AP, break_Wkp(:,Worker(indiv)%XP), cscoef_Wkp(:,:,Worker(indiv)%XP))    &
                    >= d_csval(Worker(indiv)%AP, break_Ukp, cscoef_Ukp)) then
					Worker(indiv)%EP = 1
                else
					Worker(indiv)%EP = 0
                end if
             
            else
            
                Worker(indiv)%EP = 0   
            
            end if

        end do


		!	interpolation of EmaxJGH at K_curr and Pdata(time)

		do ia = 1, na
			EmaxJGHkp(ia) = polyinterp1(K_curr, kgrid, EmaxJGH(ia,:,iPdata(time)))
		end do

		call d_csint(agrid, EmaxJGHkp, break_EmaxJ, cscoef_EmaxJ)


		!	compute firm's value of a new match

		VM = 0.0D0

		do indiv = 1, Nindiv

			if (Worker(indiv)%E == 0) then
				VM = VM + bta*d_csval(Worker(indiv)%AP, break_EmaxJ, cscoef_EmaxJ)
			end if

		end do

		VM = VM/Nind_U


		!	compute qth_0 and Thetadata

		qth_fe = kappa/VM
		theta_fe = (match_scale/qth_fe)**(1.0D0/(1.0D0-match_pow))


		!	show aggregate variables on screen

		if (mod(time,100) == 0) then
			write(*, '(A,I7)')    "time      = ", time
			write(*, '(A,F12.5)') "Pdata     = ", Pdata(time)
			write(*, '(A,F12.5)') "Kdata     = ", K_curr
			write(*, '(A,F12.5)') "Edata     = ", dble(Nind_E)/Nindiv
			write(*, '(A,F12.5)') "Udata     = ", 1.0D0-dble(Nind_E)/Nindiv
			write(*, '(A,F12.5)') "THata     = ", theta_fe
			write(*, '(A,F12.5)') "hEUdata   = ", dble(sum(Worker%E,mask=Worker%EP==0))/Nind_E
			write(*, '(A,F12.5)') "hUEdata   = ", dble(sum(Worker%EP,mask=Worker%E==0))/Nind_U
			write(*, '(A)') " "
		end if


        !   save aggregate variables for time > Nskip
        
		if (time > Nskip) then

			time2 = time - Nskip

			THdata(time2) = theta_fe
			Kdata(time2) = K_curr
			KPdata(time2) = sum(Worker%AP,mask=Worker%EP==0)/sum(1-Worker%EP,mask=Worker%EP==0)
			KEdata(time2) = sum(Worker%A,mask=Worker%E==1)/Nind_E
			Edata(time2) = dble(Nind_E)/Nindiv
			Udata(time2) = 1.0D0-Edata(time2)
			Vdata(time2) = THdata(time2)*Udata(time2)
			EUdata(time2) = dble(sum(Worker%E,mask=Worker%EP==0))/Nindiv
			UEdata(time2) = dble(sum(Worker%EP,mask=Worker%E==0))/Nindiv
			hEUdata(time2) = EUdata(time2)/Edata(time2)
			hUEdata(time2) = UEdata(time2)/Udata(time2)
			Wdata(time2) = sum(Worker%W,mask=Worker%E==1)/Nind_E


			!	save panel data

			if (panel) then

				open(100, file='Output\panel.txt', status='unknown')

				if (time2 > start_panel) then

					do indiv = 1, Nindiv_panel
				
        				if (Worker(indiv)%E == 1) then
    	        			Worker(indiv)%C = (1.0D0+irate)*Worker(indiv)%A + Worker(indiv)%W - Worker(indiv)%AP
    			        else
            				Worker(indiv)%C = (1.0D0+irate)*Worker(indiv)%A + UI - Worker(indiv)%AP
    	        		end if
						write(100, '(I3, I6, F8.3, I3, 2F7.3, I3)')                        &
						    time2-start_panel, indiv, Worker(indiv)%A, Worker(indiv)%X,     &
						    Worker(indiv)%W, Worker(indiv)%C, Worker(indiv)%E
	
					end do
			
				end if

			end if

		end if


		!	update employment status in the previous period

		Worker%E = Worker%EP
		Worker%A = Worker%AP
		Worker%X = Worker%XP
		Worker%W = 0.0D0


	end do

	close(100)


end subroutine


