!	----------------------------------------------------------------------
!	Module name : ValueModule.f90
!	----------------------------------------------------------------------	*/


module ValueModule

	use Globals
	use OptimizationMod
	use PolyInterpModule


	implicit none

	real(8):: cscoef(na,4)
	real(8):: EmaxWUFH(na,nx,nk,ne,np), EmaxWUGH(na,nk,ne,np), EUH(na,nk,ne,np)
	real(8):: EV_W(na), EV_U(na)
	real(8), external:: csval
		

contains


subroutine SolveValueFunctions()


	real(8):: NewW(na,nx,nk,ne,np), NewU(na,nk,ne,np), ae_opt, au_opt, lbd, ubd


	!	start iteration

	err_value = 1.0D0
	iter_value = 0

	do while (err_value > tol_value)


		!	compute expected values functions

		call ExpectedValueWorker()
		

		!	maximization

		do ip = 1, np
		do ie = 1, ne
		do ik = 1, nk


			!	employed worker's utility maximization

			do ix = 1, nx

				EV_W = (1.0D0-lambda)*EmaxWUFH(:,ix,ik,ie,ip) + lambda*EUH(:,ik,ie,ip)
				call csint(na, agrid, EV_W, cscoef)
	
				do ia = 1, na

					income = (1.0D0+irate)*agrid(ia) + Wage(ia,ix,ik,ie,ip)
					lbd = amin
					ubd = income
					util_leisure = 0.0D0
					call fmin(NegValueF, lbd, ubd, ae_opt)
					AE(ia,ix,ik,ie,ip) = ae_opt
					CE(ia,ix,ik,ie,ip) = income - AE(ia,ix,ik,ie,ip)
					NewW(ia,ix,ik,ie,ip) = ValueF(AE(ia,ix,ik,ie,ip))
			
				end do

			end do


			!	unemployed worker's utility maximization

			EV_U = pth(ik,ie,ip)*EmaxWUGH(:,ik,ie,ip) + (1.0D0-pth(ik,ie,ip))*EUH(:,ik,ie,ip)
			call csint(na, agrid, EV_U, cscoef)

			do ia = 1, na

				income = (1.0D0+irate)*agrid(ia) + UI
				lbd = amin
				ubd = income
				util_leisure = B_leisure
				call fmin(NegValueF, lbd, ubd, au_opt)
				AU(ia,ik,ie,ip) = au_opt
				CU(ia,ik,ie,ip) = income - AU(ia,ik,ie,ip)
				NewU(ia,ik,ie,ip) = ValueF(AU(ia,ik,ie,ip))

			end do

		end do
		end do
		end do


		!	calculate improvements of value functions

		err_W = maxval(dabs(NewW - W))
		err_U = maxval(dabs(NewU - U))
		err_value = max(err_W, err_U)


		!	update value functions

		W = sfac_value*NewW + (1.0D0-sfac_value)*W
		U = sfac_value*NewU + (1.0D0-sfac_value)*U
		
		iter_value = iter_value + 1


		!	forced exit 
		
		if (err_U <= 1.0D-1*tol_value) then
			write(*, '(A,I6/)') "Value functions have been forced to be terminated at iteration " ,iter_value
			exit
		end if
		
		
		!	show progress

		if (mod(iter_value,100) == 0 .or. (iter_value <= 20 .and. iter_wage == 0)) then
			write(*, '(A,I6)') "SolveValueFunction iteration ", iter_value
			write(*, '(2(A,D12.5)/)') "err_W = ", err_W, "   err_U = ", err_U
		end if

	end do


	write(*, '(A,I6/)') "Value functions have converged at iteration " ,iter_value
	

end subroutine



real(8) function Utility(ap)


	real(8), parameter:: tiny = 1.0D-20
	real(8):: cons, uh, ap

	cons = max(income - ap, tiny)

	if (gama == 1.0D0) then
		Utility = dlog(cons) + util_leisure
	else
		Utility = (cons**(1.0D0-gama) - 1.0D0)/(1.0D0-gama) + util_leisure
	end if


end function



real(8) function ValueF(ap)


	real(8):: ap

	ValueF = Utility(ap) + bta*csval(na, ap, agrid, cscoef)

end function



real(8) function NegValueF(ap)


	real(8):: ap

	NegValueF = -ValueF(ap)

end function



subroutine ExpectedValueWorker()


	real(8):: Ukp(na,np), maxWU(na,nx,np)


	!	compute worker's expected value functions

	EmaxWUFH = 0.0D0
	EmaxWUGH = 0.0D0
	EUH      = 0.0D0

	do ip = 1, np
	do ie = 1, ne
	do ik = 1, nk
	

		!	polynomial interpolate W and U at kp(ik,ie,ip), and compute max{W,U} in the next period

		do ia = 1, na
			
			do jp = 1, np

				Ukp(ia,jp) = polyinterp2(kp(ik,ie,ip), ep(ik,ie,ip), kgrid, egrid, U(ia,:,:,jp))
				do jx = 1, nx
					maxWU(ia,jx,jp) = max(polyinterp2(kp(ik,ie,ip), ep(ik,ie,ip), kgrid, egrid, W(ia,jx,:,:,jp)), Ukp(ia,jp))
				end do
			
			end do


			!	EmaxWUFH(ia,ix,ik,ie,ip)

			do ix = 1, nx

				do jp = 1, np
				do jx = 1, nx
					EmaxWUFH(ia,ix,ik,ie,ip) = EmaxWUFH(ia,ix,ik,ie,ip) + maxWU(ia,jx,jp)*trX(ix,jx)*trP(ip,jp)
				end do
				end do

			end do


			!	EmaxWUGH(ia,ik,ie,ip)

			do jp = 1, np
			do jx = 1, nx
				EmaxWUGH(ia,ik,ie,ip) = EmaxWUGH(ia,ik,ie,ip) + maxWU(ia,jx,jp)*prX(jx)*trP(ip,jp)
			end do
			end do


			!	EUH(ia,ik,ie,ip)

			EUH(ia,ik,ie,ip) = dot_product(Ukp(ia,:), trP(ip,:))

		end do

	end do
	end do
	end do


end subroutine



end module
