!	----------------------------------------------------------------------
!	Module name: InitializationModule.f90
!	----------------------------------------------------------------------


module InitializationModule

	use Globals
	use ValueModule
	use OptimizationMod
	use TauchenMod


	implicit none


contains


subroutine InitializeParameters()


	!	read parameters

	open(1, file='Input\Parameters.txt', status = 'old')

	read(1, *) Prod
	read(1, *) irate
	read(1, *) alpha
	read(1, *) bta
	read(1, *) gama
	read(1, *) lambda
	read(1, *) UI
	read(1, *) B_leisure
	read(1, *) kappa
	read(1, *) match_pow
	read(1, *) match_scale
	read(1, *) theta
	read(1, *) log_mean_x
	read(1, *) rhox
	read(1, *) sigex
	read(1, *) amin
	read(1, *) amax
	read(1, *) sfac_wage
	

	!	monthly interest rate and depreciation rate
	
	irate = (1.0D0+irate)**(1.0D0/12.0D0) - 1.0D0
	bta_J = 1.0D0/(1.0D0+irate)
	
	
	!	compute matching probability

	pth = match_scale*theta**match_pow
	qth = pth/theta


end subroutine




subroutine InitializeAgrid()

	implicit none

	
	!	construct unequally spaced asset grids

	step_a = (amax-amin)/(na-1)

	agrid(1) = amin
	do ia = 2, na
		agrid(ia) = agrid(ia-1) + step_a
	end do


	! save grid

	open(1, file='Output\agrid.txt', status='unknown')
	write(1, '(f12.6)') agrid
	close(1)
	
	
end subroutine



subroutine InitializeXgrid()


	!	std and mean of unconditional distribution of log_x

	sigx = sigex/dsqrt(1.0D0 - rhox**2.0D0)
	mean_log_x = log_mean_x - sigx**2.0D0/2.0D0


	!	generate grids

	lxmin = mean_log_x - xbnd*sigx
	lxmax = mean_log_x + xbnd*sigx
	lxstep = (lxmax - lxmin)/(nx-1)

	do ix = 1, nx
		lxgrid(ix) = lxmin + (ix-1)*lxstep
	end do
	xgrid = dexp(lxgrid)


	open(1, file='Output\lxgrid.txt', status='unknown')
	write(1, '(f10.6)') lxgrid
	close(1)

	open(1, file='Output\xgrid.txt', status='unknown')
	write(1, '(f10.6)') xgrid
	close(1)


end subroutine



subroutine InitializeIdiosyncraticTransition()


	real(8) prXp(nx), err_pr


	!	transition probability for x

	call TauchenTr(mean_log_x, sigx, rhox, lxgrid, trX)
	open(1, file='Output\trX.txt', status='unknown')
	write(1, '(<nx>f10.6)') ((trX(ix,jx),jx=1,nx),ix=1,nx)


	!	cumulative transition probability matrix for random drawing

	CtrX(:,1) = trX(:,1)
	do ix = 2, nx
		CtrX(:,ix) = CtrX(:,ix-1) + trX(:,ix)
	end do

	open(2, file='Output\CtrX.txt', status='unknown')
	write(2, '(<nx>f10.6)') ((CtrX(ix,jx),jx=1,nx),ix=1,nx)


	! invariant distribution of shocks

!	call TauchenNew(mean_log_x, sigx, lxgrid, prX)
!	open(3, file='Output\prX.txt', status='unknown')
!	write(3, '(f20.16)') prX


    !   new match receives the highest x
    
	prX = 0.0D0
	prX(prod_new_match) = 1.0D0
	open(3, file='Output\prX.txt', status='unknown')
	write(3, '(f20.16)') prX
    

	! cumulative invariant distribution of shocks

	CprX(1) = prX(1)
	do ix = 2, nx
		CprX(ix) = CprX(ix-1) + prX(ix)
	end do

	open(4, file='Output\CprX.txt', status='unknown')
	write(4, '(f20.16)') CprX


end subroutine



subroutine InitializeWageValueDecision()


	real(8):: betapth, Util_W, Util_U


	!	initialize value functions and Nash-bargaining wages
	!	compute Nash-bargaining wage for a assuming that a is fixed.

	betapth = 1.0D0/(1.0D0-bta*(1.0D0-lambda)+bta*pth)

	do ix = 1, nx
	do ia = 1, na

		!	initialize asset decision rules

		AE(ia,ix) = agrid(ia)
		AU(ia) = agrid(ia)


		!	solve Nash-bargaining wages given AE and AU

		Wage(ia,ix) = Prod*xgrid(ix)


		!	initialize value functions given AE, AU and Wage

		income = (1.0D0+irate)*agrid(ia) + Wage(ia,ix)
		util_leisure = 0.0D0
		Util_W = Utility(AE(ia,ix))

		income = (1.0D0+irate)*agrid(ia) + UI
		util_leisure = B_leisure
		Util_U = Utility(AU(ia))
		
		W(ia,ix) = (Util_W - bta*lambda*betapth*(Util_W-Util_U))/(1.0D0-bta)
		U(ia)    = (Util_U + bta*pth*betapth*(Util_W-Util_U))/(1.0D0-bta)
		J(ia,ix) = (Prod*xgrid(ix)-Wage(ia,ix))/(1.0D0-bta_J*(1.0D0-lambda))

	end do
	end do


end subroutine




subroutine InitializeWage()


	do ia = 1, na
		Wage(ia,:) = Prod*xgrid
	end do
	
	
end subroutine



subroutine InitializeValueDecision()


	real(8):: betapth, Util_W, Util_U


	!	initialize value functions and Nash-bargaining wages
	!	compute Nash-bargaining wage for a assuming that a is fixed.

	betapth = 1.0D0/(1.0D0-bta*(1.0D0-lambda)+bta*pth)

	do ix = 1, nx
	do ia = 1, na

		!	initialize asset decision rules

		AE(ia,ix) = agrid(ia)
		AU(ia) = agrid(ia)


		!	initialize value functions given AE, AU and Wage

		income = (1.0D0+irate)*agrid(ia) + Wage(ia,ix)
		util_leisure = 0.0D0
		Util_W = Utility(AE(ia,ix))

		income = (1.0D0+irate)*agrid(ia) + UI
		util_leisure = B_leisure
		Util_U = Utility(AU(ia))
		
		W(ia,ix) = (Util_W - bta*lambda*betapth*(Util_W-Util_U))/(1.0D0-bta)
		U(ia)    = (Util_U + bta*pth*betapth*(Util_W-Util_U))/(1.0D0-bta)
		J(ia,ix) = (Prod*xgrid(ix)-Wage(ia,ix))/(1.0D0-bta_J*(1.0D0-lambda))

	end do
	end do


end subroutine




end module
